<?php
namespace Civi\Setup\Event;

/**
 * Initialize a "Setup" subsystem. This is useful for populating the model with default settings.
 *
 * Event Name: 'civi.setup.init'
 */
class InitEvent extends BaseSetupEvent {
}
