/*
 +--------------------------------------------------------------------+
 | Copyright CiviCRM LLC. All rights reserved.                        |
 |                                                                    |
 | This work is published under the GNU AGPLv3 license with some      |
 | permitted exceptions and without any warranty. For full license    |
 | and copyright information, see https://civicrm.org/licensing       |
 +--------------------------------------------------------------------+
 */

/**
 * CiviCRM Admin Styles.
 *
 * Implements common styles on the CiviCRM admin subpages.
 *
 * @since 5.34
 */

/* CiviCRM Logo */
.wrap > img#civicrm-logo {
  float: right;
  margin-top: 9px;
}

/* Submit button on CiviCRM Settings page */
.civicrm-wrap #normal-sortables .postbox p.submit,
.civicrm-wrap #side-sortables .postbox p.submit {
  padding: 0;
  margin: 0;
  float: right;
}

/* Plugin listing on CiviCRM Integration page */
.plugin-directory-list li {
  border-bottom: 1px solid #eee;
  padding-bottom: 10px;
  margin-bottom: 10px;
}

.plugin-directory-list li:last-child {
  border-bottom: none;
  padding-bottom: 0;
  margin-bottom: 0;
}

.plugin-directory-list li span.description {
  font-style: italic;
  color: #666;
}

/* CiviCRM Quick Add dashlet */
#contact-first-name-wrap,
#contact-last-name-wrap {
  margin-bottom: 6px;
  float: left;
  width: 49%;
  margin-right: 2%;
}

#contact-last-name-wrap {
  margin-right: 0;
}

#contact-email-wrap {
  margin-bottom: 12px;
}

#side-sortables #contact-quick-add p.submit {
  float: none;
}

.contacts-added-wrap {
  border-top: 1px solid #eee;
  margin: 12px -12px 0;
  padding: 8px 12px 4px;
}

.contacts-added-wrap {
  display: none;
}

.contacts-added-wrap.contacts-added {
  display: block;
}

.contacts-added-list {
  margin-bottom: 0;
}
