<?php
namespace Civi\Setup\Event;

/**
 * Initialize the CiviCRM support files, such as `civicrm.settings.php` or `templateCompileDir`
 *
 * Event Name: 'civi.setup.installFiles'
 */
class InstallFilesEvent extends BaseSetupEvent {
}
